; Test program for Z180 CPU (C)Eugene Po - efu@mail.ru (http://orion.efu.name)
; Test RAM	- OK
; Test VGA
; Test PS/2
; Test SD card
; Test CMOS - OK
; Test Sound
; TEST LPT	- OK
; Step-7

CNTLA0	equ	00h		; ASCI Control Register A 0
CNTLB0	equ	02h		; ASCI Control Register B 0
CNTR	equ	0Ah		; CSI/O Control/Status Register
TRDR	equ	0BH		; CSI/O Transmit/Receive Data Register
TCR		equ	10H		; Timer Control Register
AECR0	equ	12h		; ASCI0 Extension Control Register
CCR		equ	1Fh		; CPU Control Register
DCNTL	equ	32h		; DMA/WAIT Control Register
;RCR	equ	36h		; Refresh Control Register
CBR		equ	38h		; MMU Common Base Register
BBR		equ	39h		; MMU Bank Base Register 
CBAR	equ	3Ah		; MMU Common/Bank Area Register
OMCR	equ	3Eh		; Operation Mode Control Register

; CPU Timing IOC = 0
;    =====================================================
;SNDADR	equ	01000110b	;  44h,  ZX 65533
SNDADR	equ	01000000b	;  44h,  ZX 65533
SNDDAT	equ	01001111b	;  46h,  ZX 49149
;SNDDAT	equ	01000100b	;  46h,  ZX 49149

SDSEL	equ	58h			;  SD   2
SDUNSEL	equ	5Ch			;  SD   2

PS2DATA	EQU	60h			;  60h  PS/2 
PS2CNTR	EQU	64h			;  64h  PS/2 

CMOSADR	equ	70h			;  70h  
CMOSDAT	equ	71h			;  71h  

PIODRA	equ	01111000b	;  78h PIO A Data register
PIODRB	equ	01111001b	;  79h PIO B Data register
PIOCRA	equ	01111010b	;  7Ah PIO A Control register
PIOCRB	equ	01111011b	;  7Bh PIO B Control register

VIDEO_RAM	equ 08000h	;  
;==============================================================================
  org (0000h)
START:
  di				; . 
  xor a				;  
  out0 (36h),a		; .  
  in0	a,(OMCR)
  res	5,a
  out0	(OMCR),a	; IO   Zilog
;  out0 (TCR),a		; . , A18 - 
;  ld a,00110000b
;  out0 (DCNTL),a	; .  
;  ld a,00010111b
;  out0 (CCR),a		;  33%
;=============================================================================
;   Bank Area = 8000H, Common Area 1 = C000h
;=============================================================================  
  ld a,0C8H		;  Common Area-1 - 0C000h, Bank Area - 08000h
  out0 (CBAR),a
;  ld a,0A0H		; VGA    0A0000h   Bank Area
;  ld a,0B8H		; VGA    0B8000h   Bank Area
  ld a,80H		; RAM  80000h   Bank Area
  out0 (BBR),a
  ld a,10H		; RAM  10000H   Common Area-1
  out0 (CBR),a
;=============================================================================
;  Serial Port 0
;=============================================================================
;  in0 a,(AECR0)
;  set 4,a
;  out0 (AECR0),a	;    ASCI0 18,432 
;  set 7,a		;    PHI = 18,432 
;  out0 (CCR),a		; CPU Control Register
  ld a,01110100b	;    . RTS=1,8,N,1
  out0 (CNTLA0),a	; Init CNTLA0 (COM1)
;  ld a,10111000b	; PS=30, DR=64, SS=1, 9600BPS  PHI=18,432   4800  PHI/2
  ld a,10110000b	; PS=30, DR=16, SS=1, 9600BPS  PHI=18,432/4/2  = 2.304 
;  ld a,10111111b	; DR=64, SS=111, 28800BPS    1,8432 
  out0 (CNTLB0),a	; Init CNTLB0 (COM1)
;==============================================================================
; Test RAM0
;==============================================================================
TEST_RAM0:
  ld	hl,0C000h	;  
  ld	a,(hl)		;  
  cpl				; 
  ld	(hl),a		;   (  )
  ld	a,(hl)		;   
  cp	(hl)		; 
  jr	z,RAM0_OK	;  ,   OK,  ERR
  ld	hl,RAM0_ERR_String	;    "RAM0 Err!"
out_str1:
  ld	a,(hl)
  cp	0		;     "00"
  jr	z,TEST_RAM_VGA
  ld	c,a
test_sym1:
  in0	a,(04h)		;    COM1 (STAT0)
  rra
  rra			;   1
  jr	nc,test_sym1	;     -    ( C=0),  
  out0	(06h),c		;   COM1
  inc	hl		;  
  jr	out_str1
;    RAM0 - RAM VGA  
TEST_RAM_VGA:
  ld hl,RAMVGA_ERR_String
  jp STOP
;==============================================================================
RAM0_OK:
  ld	sp,0D000h			;  
  ld	hl,R_String			;  -
  call	out_string
  ld	hl,WelcomeString	;   
  call	out_string
;  ld	b,7
;  call	beep				; 
  ld	hl,RAM0_OK_String	;   "RAM0 OK"
  call	out_string
;==============================================================================
; Test RAM1
;==============================================================================
TEST_RAM1:
  ld	hl,08000h		;  
  ld	a,(hl)			;  
  cpl					; 
  ld	(hl),a			;   (  )
  ld	a,(hl)			;   
  cp	(hl)			; 
  jr	z,RAM1_OK		;  ,   OK,  ERR
  ld	hl,RAM1_ERR_String	;   "RAM1 Err!"
  call	out_string
;  ld	b,5
;  call	beep
  jr	INIT_HW
RAM1_OK:
;  ld	b,7
;  call	beep			; 
  ld	hl,RAM1_OK_String	;   "RAM1 OK"
  call	out_string
;=============================================================================
; Test VGA RAM
;=============================================================================
TEST_RAM_VGA1:

;=============================================================================
INIT_HW:
	call	INIT_CMOS	;    
	call	beep5
	call	INIT_PS2	;  PS/2
	call	beep1
;	call	INIT_VGA	;  VGA
	call	beep2
;	call	INIT_CSIO	;  SPI SD Card
	call	beep4
;	call	INIT_SND	;   
	call	beep2
	call	INIT_PIO	;   
	call	beep1
;==============================================================================
TEST:
  call	in_sym
;  cp	0FFh
;  call	z,KEYB_IN
  cp	'c'
  call	z,TEST_CMOS
  cp	'f'
  call	z,TEST_CSIO
  cp	'h'
  jr	z,IOC_HITACHI
  cp	'k'
  call	z,TEST_PS2
  cp	'l'
  call	z,KEYB_LED_ON
  cp	'm'
  call	z,TEST_MEM
  cp	'p'
  call	z,TEST_PIO
  cp	'r'
  jp	z,START
  cp	's'
  call	z,TEST_SND
  cp	't'
  call	z,SET_CMOS
  cp	'o'
  call	z,KEYB_LED_OFF
  cp	'v'
  call	z,TEST_VGA
  cp	'z'
  jr	z,IOC_ZILOG
  
  cp	'1'
  call	z,INIT_VGA
  cp	'2'
  call	z,INIT_PS2
  cp	'3'
  call	z,INIT_CSIO
  cp	'4'
  call	z,INIT_CMOS
  cp	'5'
  call	z,INIT_SND
  cp	'6'
  call	z,INIT_PIO
  cp	'7'
  call	z,KEYB_IN
  cp	0FFh
  jr	z,TEST
  call	out_hex		;   
  ld	hl,SYM_CR_LF
  call	out_string	;  
;  ld	a,'_'
;  call	out_sym		;  
  
  jr TEST
;==============================================================================
IOC_HITACHI:
  in0	a,(OMCR)
  set	5,a
  out0	(OMCR),a
  ld	b,4
  call	beep
  jp	TEST
IOC_ZILOG:
  in0	a,(OMCR)
  res	5,a
  out0	(OMCR),a
  ld	b,8
  call	beep
  jp	TEST
WaitFF:
  ld a,0F0h
  out0 (DCNTL),a	; .  
  jp TEST
Wait00:
  ld a,00h
  out0 (DCNTL),a	; .  
  jp TEST
;==============================================================================
ReadDataKeyb:
  ld bc,PS2CNTR		;      bc
  in a,(c)		;   
  rra			;   
  jp nc,ReadDataKeyb	;    ( 0 = 0)-  
  ld bc,PS2DATA		;     bc
  in a,(c)		;   A
  ld hl,SCAN_CODE_SET1	;  
  add a,l
  ld l,a
  ld a,(hl)		;   
  call out_sym		;   
  jp ReadDataKeyb
;==============================================================================
STOP:
  set 7,a			;   7
  out0 (36h),a		; .   , .. 
  ld hl,StopString	; !!!
out_str2:
  ld	a,(hl)
  cp	0			;     "00H"
  jr	z,SYSTEM_STOP
  ld	c,a
test_sym2:
  in0	a,(04h)		;    COM1 (STAT0)
  rra
  rra				;   1
  jr	nc,test_sym2	;     -    ( C=0),  
  out0	(06h),c			;   COM1
  inc	hl
  jp	out_str2
SYSTEM_STOP:
  halt			; !!!
;==============================================================================
; 
;==============================================================================
;        A
;==============================================================================
in_sym:
  in0	a,(04h)		;    COM1 (STAT0)
  rla				;   7
  jr	nc,NO_IN	;     ( C=0),   A 0FFh
  in0	a,(08h)		;   COM1
  ret
NO_IN:
  ld	a,0FFh
  ret
;==============================================================================
; Test VGA -   VGA    
;==============================================================================
TEST_VGA:
  ld	b,63		;  
  ld	hl,VIDEO_RAM	;  
test_vga1:
  ld	a,(hl)		;  
  call	out_hex		;   
  ld	a,'_'
  call	out_sym		;  
  inc	hl
  djnz	test_vga1
  ret
;=============================================================================
;  VGA
;==============================================================================
INIT_VGA:
  ld	a,0B8H		; VGA    0B8000h   Bank Area
  out0	(BBR),a
  ld a,1		;       /  
  ld bc,3C3h
  out (c),a
  
  ld	hl,08000h		;  
  ld	a,(hl)			;  
  cpl				; 
  ld	(hl),a			;   (  )
  ld	a,(hl)			;   
  cp	(hl)			; 
  jr	z,RAM_VGA_OK		;  ,   OK,  ERR
  ld	hl,RAM_VGA_ERR_String	;   "RAM VGA Error"
  call	out_string
  jr	INIT_VGA_1
RAM_VGA_OK:
  ld	hl,RAM_VGA_OK_String	;   "RAM VGA OK"
  call	out_string
INIT_VGA_1:  
  ld a,23h		; 
  ld bc,3C2h		;      MOR
  out (c),a		;
  ld bc,3CCh		;  ,     VGA MOR
  in a,(c)
  cp 23h
  jp z,VGA_OK		;  ,  VGA .
  ld hl,VGA_ERR_String	;   ERR
  call out_string
;  ret
VGA_OK:
  ld bc,3DAh		;   3DA
  in a,(c)

  call beep		; 
  ld hl,VGA_OK_String	;   OK
  call out_string
  
  ld hl,REGS_DATA	;   
  ld a,(hl)		;   
  ld bc,3C2h		;     
  out (c),a
  inc hl
  ld a,(hl)		;   
  ld bc,3DAh		;    
  out (c),a
  inc hl
;=============================================================================
;   SEC 5 .
;=============================================================================
  xor a
VGA_SEC:
  ld bc,3C4h
  out (c),a		;   A
  ld d,(hl)		;   
  ld bc,3C5h
  out (c),d		;   D
  inc hl  		;  
  cp 5
  inc a			;   
  jp c,VGA_SEC		;    5 
;=============================================================================
;   CRTC 25 . - 18H
;=============================================================================
  xor a
VGA_CRT:  
  ld bc,3D4h
  out (c),a
  ld d,(hl)		;   
  ld bc,3D5h
  out (c),d
  inc hl  		;  
  cp 18h
  inc a			;   
  jp c,VGA_CRT		;    25 
;=============================================================================
;    GC 9 .
;=============================================================================
  xor a
VGA_GC:
  ld bc,3CEh
  out (c),a
  ld d,(hl)		;   
  ld bc,3CFh
  out (c),d
  inc hl  		;  
  cp 9
  inc a			;   
  jp c,VGA_GC		;    9 
;=============================================================================
;    AC 21 .
;=============================================================================
  xor a
VGA_AC:
;  ld bc,3BAh
;  in d,(c)		;        
  ld bc,3DAh
  in d,(c)		;        
  ld bc,3C0h
  out (c),a
  ld d,(hl)		;   
  ld bc,3C0h
  out (c),d
  inc hl  		;  
  cp 21
  inc a			;   
  jp c,VGA_AC		;    21 
;=============================================================================
;    2  
;=============================================================================
  ld bc,03c4h	;      (Color Plane Write Enable - CPWE) ( 2)
  ld a,2
  out (c),a
  ld bc,03c5h
  ld a,0100b	;   D2  ,        .
  out (c),a
;  ld hl,FONT
  ld de,VIDEO_RAM
  ld bc,4095
  ldir		;  
  ld bc,03c4h	;      (Color Plane Write Enable - CPWE) ( 2)
  ld a,2
  out (c),a
  ld bc,03c5h
  ld a,1	;        
  out (c),a
;=============================================================================
;     VGA
;=============================================================================
  ld hl,Welcome_string
  ld de,VIDEO_RAM
Print_String:
  ld a,(hl)
  ld (de),a
  cp 0dh
  inc hl
  inc de
  inc de
  jp c,Print_String
  call beep		; 
  ld hl,VGA_INIT_String	;   INIT
  call out_string
  ret
;==============================================================================
;  PS/2 
;==============================================================================
TEST_PS2:
;  call	KEYB_IN
;  call	out_hex
;  ld	a,'.'
;  call	out_sym
  in	a,PS2CNTR		;   
  rra
  rra				;   
  ret	c			;    ( 1 = 1) 
  ld	a,0C0h
  out	PS2CNTR,a
  in	a,PS2DATA		;   
  call	out_hex
  ld	a,'.'
  call	out_sym
  ld	a,0D0h
  out	PS2CNTR,a
  in	a,PS2DATA		;   
  call	out_hex
  ld	hl,SYM_CR_LF
  call	out_string		;  
  ret
;==============================================================================
;  PS/2 
;==============================================================================
INIT_PS2:
  in	a,PS2CNTR			;   
  rra
  rra						;   
  jr	c,KEYB_NOT_READY	;    ( 1 = 1)
  
  ld 	a,0AAH
  out	PS2CNTR,a		;  
  ld 	d,0FFh			;  ff 
KEYB_TEST:
  dec 	d
  ld 	a,d
  jr 	z,KEYB_NOT_READY
  in 	a,PS2CNTR		;   
  rra
  rra					;   
  jr 	c,KEYB_TEST		;    ( 1 = 1)-  
  in	a,PS2DATA		;   
  cp	055H
  jr	nz,KEYB_ERROR
  ld	hl,KeybOkString
  call	out_string		; Test OK -   
  call	beep2
  ret
  
KEYB_ERROR:
  cp 0FDH
  ld hl,KeybErrorString
  call z,out_string		; Diag. Failure -   .
  call beep2
  ret
  
KEYB_NOT_READY:
  ld hl,KeybNRstring	;   
  call out_string
  call beep5
  ret
;==============================================================================
;     PS/2
;==============================================================================
KEYB_IN:
	in	a,PS2CNTR	;   
;	rra
	rra				;   
	jp	nc,KEYB_IN
	in	a,PS2DATA
	ld	b,a
KEYB_IN_1:	
	in	a,PS2CNTR
;	rra
	rra
	rra
	jp	c,KEYB_IN_1
	ld	a,0ADh			;  AD
	out	PS2CNTR,a
KEYB_IN_2:
	in	a,PS2CNTR
;	rra
	rra
	rra
	jp	c,KEYB_IN_2
	ld	a,0AEh			;  AE
	out	PS2CNTR,a
	ld	a,b
	ret
;==============================================================================
;     PS/2
;==============================================================================
KEYB_LED_ON:
	in		a,PS2CNTR		;   
	rra
	rra						;   
	jp		c,KEYB_LED_ON
	ld		a,0EDh
	out		PS2DATA,a		;   
KEYB_LED_ON_1:
	in		a,PS2CNTR		;   
;	rra
	rra
	jp		nc,KEYB_LED_ON_1
	in		a,PS2DATA
	cp		0FAh
	jp		nz,KEYB_LED_ON_ERR
	ld		a,07h
	out		PS2DATA,a		;   
KEYB_LED_ON_2:
	in		a,PS2CNTR		;   
;	rra
	rra						;   
	jp		nc,KEYB_LED_ON_2
	in		a,PS2DATA
	cp		0FAh
	jp		nz,KEYB_LED_ON_ERR
	call	beep1
	ret
KEYB_LED_ON_ERR:
	ld		hl,KeybLedOnErrString
	call	out_string		;   
	call	beep5
	ret
;==============================================================================
;     PS/2
;==============================================================================
KEYB_LED_OFF:
	in		a,PS2CNTR		;   
	rra
	rra						;   
	jp		c,KEYB_LED_ON
	ld		a,0EDh
	out		PS2DATA,a		;   
KEYB_LED_OFF_1:
	in		a,PS2CNTR		;   
;	rra
	rra
	jp		nc,KEYB_LED_OFF_1
	in		a,PS2DATA
	cp		0FAh
	jp		nz,KEYB_LED_OFF_ERR
	ld		a,00h
	out		PS2DATA,a		;   
KEYB_LED_OFF_2:
	in		a,PS2CNTR		;   
;	rra
	rra						;   
	jp		nc,KEYB_LED_OFF_2
	in		a,PS2DATA
	cp		0FAh
	jp		nz,KEYB_LED_OFF_ERR
	call	beep1
	ret
KEYB_LED_OFF_ERR:
	ld		hl,KeybLedOnErrString
	call	out_string		;   
	call	beep5
	ret
;==============================================================================
; ,    B
;==============================================================================
beep5:
  ld	b,00100111b
  jp	beep
beep1:
  ld	b,00100110b
  jp	beep
beep2:
  ld	b,00100101b
  jp	beep
beep4:
  ld	b,00100100b
beep:
  ld	a,0Ah
  out	(CMOSADR),a
  ld	a,b
  out	(CMOSDAT),a		;     SQW
  ld	a,0Bh
  out	(CMOSADR),a
  in	a,(CMOSDAT)
  set	3,a				;   3 . 
  ld	b,a				;  A
  ld	a,0Bh
  out	(CMOSADR),a
  ld	a,b				;  A
  out	(CMOSDAT),a		;  
  
 
  ld	b,10h			;   
beep_wait:
  push	bc
  ld	b,0FFh			;  
beep_wait1:
  nop  
  nop
  nop
  nop
  nop
  nop
  djnz	beep_wait1
  pop	bc
  djnz	beep_wait

  ld	a,0Ah
  out	(CMOSADR),a
  ld	a,00100000b
  out	(CMOSDAT),a		;   - =0

  ret
;==============================================================================
;    A  HEX 
;==============================================================================
out_hex:
  PUSH	AF
  RRCA
  RRCA
  RRCA
  RRCA
  CALL	NIBBLE
  POP	AF
NIBBLE:
  AND	0FH			;   4 
  add	a,30h		;  30h
  CP	3Ah			;     "9"?
  jr	c,out_hex_1
  add	a,7			;  7   A-F
out_hex_1:
  call	out_sym		;   
  ret
;==============================================================================
;    HL    0
;==============================================================================
out_string:  
  ld	a,(hl)
  cp	0			;     "00H"
  ret	z
  call	out_sym
  inc	hl
  jp	out_string
;==============================================================================
;    A  
;==============================================================================
out_sym:
  ld	c,a
test_sym:
  in0	a,(04h)		;    COM1 (STAT0)
  rra
  rra			;   1
  jr	nc,test_sym	;     -    ( C=0),  
  out0	(06h),c		;   COM1
  ret
;=============================================================================
;  CMOS
;=============================================================================
INIT_CMOS:
  ld	a,0Dh		;  D
  out	(CMOSADR),a
  in	a,(CMOSDAT)
  rla
  jr	c,CMOS_OK
CMOS_ERROR:			;  
  ld	hl,CMOS_ERR_String
  call	out_string
  ret
CMOS_OK:			;  
  ld	a,0Ah
  out	(CMOSADR),a
  ld	a,00100000b
  out	(CMOSDAT),a	; . 
  ld	a,0Bh
  out	(CMOSADR),a
  ld	a,00001010b	; 
  out	(CMOSDAT),a	;  
  ld	hl,CMOS_OK_String
  call	out_string
;  ld	b,4
;  call	beep
  ret
;=============================================================================
;  CMOS
;=============================================================================
TEST_CMOS:
  ld a,0Ah
  out (CMOSADR),a
  in a,(CMOSDAT)
  rla
  jr	c,TEST_CMOS	;   
  ld a,4h			;  
  out (CMOSADR),a
  in a,(CMOSDAT)
  call out_hex		;   
  ld a,':'
  call out_sym		;  
  ld a,2h			;  
  out (CMOSADR),a
  in a,(CMOSDAT)
  call out_hex		;   
  ld a,':'
  call out_sym		;  
  ld a,0h			;  
  out (CMOSADR),a
  in a,(CMOSDAT)
  call out_hex		;   
  ld hl,SYM_CR_LF
  call out_string	;  

  ret

  ld d,0eh		;      0E  7F
Read_CMOS:
  ld a,d
  out (CMOSADR),a
  in a,(CMOSDAT)
  call out_hex		;   
  ld a,' '
  call out_sym		;  
  inc d
  ld a,d
  cp 80h
  ret z
  jr Read_CMOS
;=============================================================================
;   CMOS
;=============================================================================
SET_CMOS:
  ld	a,0Ah
  out	(CMOSADR),a
  ld	a,00100110b
  out	(CMOSDAT),a		;  1024    SQW
  ld	a,0Bh
  out	(CMOSADR),a
  ld	a,11111010b
  out	(CMOSDAT),a		;   
  ld	a,4
  out	(CMOSADR),a
  ld	a,1
  out	(CMOSDAT),a		; 
  ld	a,2
  out	(CMOSADR),a
  ld	a,1
  out	(CMOSDAT),a		; 
  ld	a,0
  out	(CMOSADR),a
  ld	a,1
  out	(CMOSDAT),a		; 
  
  ld	hl,TIME_SET_String	;   "Time Setting"
  call	out_string

  ld	a,0Bh
  out	(CMOSADR),a
  ld	a,01111010b
  out	(CMOSDAT),a		;  

  ld	a,0FFh
  ret
;=============================================================================
;  RAM   64 
;=============================================================================
TEST_MEM:
  ld	hl,SYM_CR_LF		;     
  call	out_string
  in0	a,(BBR)	
  ld	b,a			;  BBR
  ld	a,10H

TEST_MEM1:
  out0 (BBR),a
  push	af
  call out_hex
  ld	hl,08000h		;  
  ld	a,(hl)			;  
  cpl				; 
  ld	(hl),a			;   (  )
  ld	a,(hl)			;   
  cp	(hl)			; 
  jr	z,MEM_OK		;  ,   OK,  ERR

  ld	hl,RAM_ERR_String	;   "RAM Error"
  call	out_string
  pop	af
  add	a,10H
  out0 (BBR),b			;  BBR
  ret	c
  jp	TEST_MEM1
MEM_OK:
  ld	hl,RAM_OK_String	;   "RAM OK"
  call	out_string
  pop	af
  add	a,10H
  out0 (BBR),b			;  BBR
  ret	c
  jp	TEST_MEM1
;=============================================================================
;  LPT
;=============================================================================
TEST_PIO:
  ld	b,25h
  call	beep
  
  ld	hl,WelcomeString
out_string_prn:  
  ld	a,(hl)
  cp	0			;     "00"
  ret	z
  call	out_sym_prn
  inc	hl
  jp	out_string_prn
;==============================================================================
;    A  
;==============================================================================
out_sym_prn:
;  ld	c,a
test_sym_prn:
;  in	a,(PIOCRA)	;   PIO-A port
;  in0	a,(04h)		;    COM1 (STAT0)
;  rra
;  rra			;   1
;  jr	nc,test_sym_prn	;     -    ( C=0),  
  out	(PIODRA),a	;   
  ret
;=============================================================================
;  PIO -   - !!!
;=============================================================================
INIT_PIO:
  ld	a,0fh		; Output mode 0
  out	(PIOCRA),a
  out	(PIOCRB),a
  ld	a,07h		;  
  out	(PIOCRA),a
  out	(PIOCRB),a
  ld	a,03h		;  
  out	(PIOCRA),a
  out	(PIOCRB),a
  in	a,(PIODRA)	;  
  cpl			; 
  ld	b,a
  out	(PIODRA),a	; 
  in	a,(PIODRA)	;  
  cp	b
  jr	z,PIO_OK
  ld	hl,PIO_ErrString
  call	out_string
  call	beep
  call	beep
  ret
PIO_OK:  
  ld	hl,PIO_OKString
  call	out_string
  call	beep
  ret
;=============================================================================
;    Yamaha
;  : Z  -  .    - .
;=============================================================================
INIT_SND:
  XOR   A          	; A=0
  OUT   (SNDADR),A    ;   0
  OUT   (SNDDAT),A    ;  0   
  IN    A,(SNDADR)	;     
  or	a
  jr	nz,SND_ERR
  ld	hl,SoundOKString	;  
  call	out_string
  call	beep
  ret
SND_ERR:
  ld	hl,SoundErrString	;   
  call	out_string
  call	beep
  call	beep
  call	beep
  RET
;=============================================================================
;    Yamaha -  
; https://zxpress.ru/book_articles.php?id=667
;=============================================================================
TEST_SND:
  ld	b,7
  call	beep

	LD A,3        	;  65533   3- 
	OUT (SNDADR),A  ;(     )
	LD A,5        	;(  5)
	OUT (SNDDAT),A     ;

	LD A,7        ;
	OUT (SNDADR),A	;  7
	LD A,%11111101	;(  ,  
	OUT (SNDDAT),A     ;
	
	LD A,9        	;(  )
	OUT (SNDADR),A  ;  9
	LD A,16       	;(..   16,  -
	OUT (SNDDAT),A  ; 
	
	LD A,12       	;(   )
	OUT (SNDADR),A  ;  12
	LD A,255      	;  
	OUT (SNDDAT),A     ;
	
	LD A,13       	;  
	OUT (SNDADR),A     ;
	LD A,%11111110;
	OUT (SNDDAT),A     ;
	
	ld	hl,SoundTestString	;  
	call	out_string
	ret
;==============================================================================
;  Serial Port CSI/O - SD 
;==============================================================================
INIT_CSIO:
  xor	a
  out	(SDSEL),a		;  SD card
  ld	a,00001000b		;  SS=20
  out0	(CNTR),a		;     CSI/O   18,432/4/2/20 = 115,2 
;  ld	a,0		
  ld	hl,CMD0_data		;   CMD0 -  
  call	out_csio
  call	beep			; 
  call	in_csio			;  
  jr	z,CSIO_OK
  ld	hl,CSIO_Err_String	; SD   !!!
  call	out_string
  call	beep
  call	beep
  ret
CSIO_OK:			; SD  
;  ld	a,8	
  ld	hl,CMD8_data		;   CMD8
  call	out_csio
  call	beep			; 
  call	in_csio
  call	out_hex

  ld	hl,CSIO_OK_String	; SD  
  call	out_string
  ret
;==============================================================================
in_csio:		;   SD 
  in0	a,(CNTR)
  set	5,a
  res	4,a
  out0	(CNTR),a	;   CSI/O   
  in0	a,(TRDR)	;  SD
  ret
;==============================================================================
out_csio:		;  6   CRC  SD     hl
  ld	b,5
  in0	a,(CNTR)
  set	4,a
  out0	(CNTR),a	;   CSI/O
out_scio1:
  ld	a,(hl)
  out0	(TRDR),a	; 
  inc	hl
  djnz	out_scio1
  ret
;==============================================================================
CMD0_data:
  .db 40h, 00, 00, 00, 00, 95h	; 48 ,   0x95
CMD8_data:
  .db 48h, 00, 00, 00, 00, 95h
;==============================================================================
;    
; . 149 datasheet
;==============================================================================
TEST_CSIO:
  xor	a
  out	(SDSEL),a		;  SD card

;  call	beep		; 
;  in0	a,(CNTR)
;  call	out_hex		;   

  ld	hl,SYM_CR_LF
  call	out_string	;  
  call	beep

  in0	a,(CNTR)
  set	4,a
  out0	(CNTR),a	;   CSI/O
  xor	a
  out0	(TRDR),a	;  0
  call	beep		; 
  in0	a,(CNTR)
  set	5,a
  out0	(CNTR),a	;   CSI/O
  in0	a,(TRDR)	; 
  call	out_hex		;   
  ld	hl,SYM_CR_LF
  call	out_string	;  
  ld	a,0FFh
  out	(SDSEL),a	;  SD card
  ret

;SCAN_CODE_SET2:
; .db 0,0,0,0,0,0,0,0,0,0,0,0,0,126,0,0,0,0,0,0,'Q','1',0,0,0,'Z','S','A','W','2',0,0,'C','X','D','E','4','3',0,0,' ','V"
; .db "F','T','R','5',0,0,'N','B','H','G','Y','6',0,'J',0,'M',0,'U','7','8',0,0,44,'K','I','O','0','9',0,0,'.','/','L',59,'P','-',0,0,0
; .db 39,0,'[','=',0,0,0,0,0,']',0,92,0,0,0,0,0,0,0,0,0,0,0,'1',0,'4','7',0,0,0,'0','.','2','5','6','8',0,0,'0','+','3','-','*','9',0,0,0,0,0,0
;=============================================================================
REGS_DATA:
; MISC
; .db    67h, 00h
.db    10100111b, 00h	; 400 x 720
; SEQ
.db    03h, 00h, 03h, 00h, 02h
; CRTC
.db    5Fh, 4Fh, 50h, 82h, 55h, 81h, 0BFh, 01Fh
.db    00h, 4Fh, 0Dh, 0Eh, 00h, 00h, 00h, 50h
.db    9Ch, 0Eh, 8Fh, 28h, 1Fh, 96h, 0B9h, 0A3h
.db    0FFh
; GC
.db    00h, 00h, 00h, 00h, 00h, 10h, 0Eh, 00h
.db    0FFh
; AC
.db    00h, 01h, 02h, 03h, 04h, 05h, 14h, 07h
.db    38h, 39h, 3Ah, 3Bh, 3Ch, 3Dh, 3Eh, 3Fh
.db    0Ch, 00h, 0Fh, 08h, 00h
;=============================================================================
#include	"messages.asm"
; #include	"fontvga.asm"

end
